# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_context.ipynb.

# %% auto 0
__all__ = ['fetch_site_from_sitemap']

# %% ../nbs/01_context.ipynb 2
import os, re, requests
from fastcore.script import call_parse, store_true, Param
from concurrent.futures import ThreadPoolExecutor, as_completed
from urllib.parse import urlparse, urljoin
import pandoc
from pathlib import Path
from requests.exceptions import RequestException
import xml.etree.ElementTree as ET
from bs4 import BeautifulSoup
from tqdm import tqdm

# %% ../nbs/01_context.ipynb 3
def _sanitize_filename(name):
    invalid_chars = '<>:"/\\|?*'
    for char in invalid_chars:
        name = name.replace(char, '_')
    # Strip existing file extension if any when markdown is expected
    name = os.path.splitext(name)[0]
    return name

def _html_to_markdown_pandoc(html_content, strip_html=False, media_pth=None):
    p = pandoc.read(source=html_content, format='html')
    md = pandoc.write(p, format='markdown_strict') # options=['--extract-media', media_pth]
    if strip_html: md = re.sub(r'<[^>]*>', '', md)
    return md

def _fetch_and_save(url:str, output_dir:str, to_markdown:bool, strip_html:bool):
    try:
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        content = response.content  # Assuming binary content to handle non-text files as well

        parsed_url = urlparse(url)
        filename = _sanitize_filename(parsed_url.path.strip('/').replace('/', '_') or 'index')
        media_dir = str(Path(output_dir)/(filename+ '_media'))
        # Determine the file extension based on to_markdown flag
        file_extension = '.md' if to_markdown else '.html'
        filename += file_extension

        filepath = os.path.join(output_dir, filename)
        os.makedirs(os.path.dirname(filepath), exist_ok=True)

        with open(filepath, 'wb' if not to_markdown else 'w', encoding=None if not to_markdown else 'utf-8') as f:
            if to_markdown:
                # Convert HTML to markdown if needed
                content = _html_to_markdown_pandoc(content.decode('utf-8'), media_pth=media_dir, strip_html=strip_html)
            f.write(content)
        return url, None
    except RequestException as e:
        return url, f"HTTP error for URL '{url}': {e}"
    except Exception as e:
        return url, f"Failed to fetch or save URL '{url}': {e}"


def _is_valid_url(url, domain):
    """Check if a URL belongs to the specified domain and is a valid fetch target."""
    parsed_url = urlparse(url)
    return parsed_url.netloc == domain and parsed_url.scheme in ['http', 'https']


def _extract_links(url):
    """
    Extract all hyperlinks from the specified webpage URL.

    Parameters:
    - url (str): The URL of the webpage from which to extract links.

    Returns:
    - set: A set containing all the unique hyperlinks found on the webpage.
    """
    links = set()
    try:
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        soup = BeautifulSoup(response.content, 'html.parser')
        for link in soup.find_all('a', href=True):
            absolute_link = urljoin(url, link['href'])
            links.add(absolute_link)
    except RequestException as e:
        print(f"HTTP error for URL '{url}': {e}")
    except Exception as e:
        print(f"Failed to extract links from URL '{url}': {e}")
    return links

# %% ../nbs/01_context.ipynb 4
@call_parse
def fetch_site_from_sitemap(url:str, # The site that you want to fetch documents from 
                            output_dir:str, # The output directory to write documents to, 
                            to_html:Param('Write output as HTML instead of markdown.', store_true),
                            strip_html:Param('Strip HTML artifacts out of markdown.', store_true)
                           ) :
    """
    Fetch content from the specified URL and save it to the given directory. Saves the content as a markdown file unless `to_html` is True. 
    """
    
    to_markdown = not to_html
    sitemap_url = urljoin(url, '/sitemap.xml')
    domain = urlparse(url).netloc
    
    def crawl_and_fetch(base_url):
        visited = set()
        to_visit = set([base_url])
    
        with tqdm(total=1, unit='file') as pbar:  # Initialize tqdm with an arbitrary total
            while to_visit:
                current_url = to_visit.pop()
                if current_url not in visited:
                    visited.add(current_url)
                    links = _extract_links(current_url)
                    valid_links = {link for link in links if _is_valid_url(link, domain)}
                    to_visit.update(valid_links - visited)  # Only add links that haven't been visited
                    _fetch_and_save(current_url, output_dir, to_markdown, strip_html)
                    pbar.update(1)  # Update progress bar for each URL processed
                pbar.total = len(visited) + len(to_visit)  # Update the total count
                pbar.refresh() 

    try:
        sitemap_response = requests.get(sitemap_url, timeout=10)
        sitemap_response.raise_for_status()
        sitemap_xml = sitemap_response.text
        # Process sitemap as before...
    except RequestException:
        print(f"Sitemap not found, starting crawl at {url}")
        crawl_and_fetch(url)
    except Exception as e:
        print(f"Unexpected error: {e}")
        return

    try:
        root = ET.fromstring(sitemap_xml)
        namespaces = {'ns': root.tag.split('}')[0].strip('{')}
        urls = [url.text for url in root.findall('ns:url/ns:loc', namespaces)]
    except ET.ParseError as e:
        print(f"Error parsing the XML sitemap: {e}")
        return
    except Exception as e:
        print(f"Unexpected error when parsing sitemap: {e}")
        return

    os.makedirs(output_dir, exist_ok=True)

    tasks = []
    with ThreadPoolExecutor() as executor:
        for url in urls:
            tasks.append(executor.submit(_fetch_and_save, url, output_dir, to_markdown, strip_html))

        for future in tqdm(as_completed(tasks), total=len(tasks), unit='file'):
            url, error = future.result()
            if error:
                print(error)



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/hamel',
                'doc_host': 'https://hamelsmu.github.io',
                'git_url': 'https://github.com/hamelsmu/hamel',
                'lib_path': 'hamel'},
  'syms': { 'hamel.context': { 'hamel.context._extract_links': ('context.html#_extract_links', 'hamel/context.py'),
                               'hamel.context._fetch_and_save': ('context.html#_fetch_and_save', 'hamel/context.py'),
                               'hamel.context._html_to_markdown_pandoc': ('context.html#_html_to_markdown_pandoc', 'hamel/context.py'),
                               'hamel.context._is_valid_url': ('context.html#_is_valid_url', 'hamel/context.py'),
                               'hamel.context._sanitize_filename': ('context.html#_sanitize_filename', 'hamel/context.py'),
                               'hamel.context.fetch_site_from_sitemap': ('context.html#fetch_site_from_sitemap', 'hamel/context.py')},
            'hamel.oai': { 'hamel.oai._join_url': ('oai.html#_join_url', 'hamel/oai.py'),
                           'hamel.oai._query_git_config': ('oai.html#_query_git_config', 'hamel/oai.py'),
                           'hamel.oai._to_dt': ('oai.html#_to_dt', 'hamel/oai.py'),
                           'hamel.oai.create_openai_plugin_scaffolding': ('oai.html#create_openai_plugin_scaffolding', 'hamel/oai.py'),
                           'hamel.oai.list_models': ('oai.html#list_models', 'hamel/oai.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_oai.ipynb.

# %% auto 0
__all__ = ['list_models', 'create_openai_plugin_scaffolding']

# %% ../nbs/00_oai.ipynb 3
from datetime import datetime
import inspect, os, json, subprocess
import pandas as pd
from fastcore.script import call_parse
from fastcore.meta import delegates
from urllib.parse import urljoin

# %% ../nbs/00_oai.ipynb 4
def _to_dt(dt:int):
    return datetime.fromtimestamp(dt).strftime('%Y-%m-%d')

@call_parse
def list_models(owned:str=None, # Filter by who models are owned by
                limit:int=None # Limit results to the most n created models.
               ):
    "List OpenAI models you have access to."
    from openai import OpenAI
    client = OpenAI()
    models = list(client.models.list())
    models.sort(key=lambda m: -m.created)
    models = [{'Name': m.id, 'Created Dt': _to_dt(m.created)} for m in models if not owned or owned in m.owned_by]
    if limit: models = models[:limit]
    df = pd.DataFrame(models)
    return df.to_markdown(index=False)

# %% ../nbs/00_oai.ipynb 6
def _query_git_config(key: str):
    try:
        result = subprocess.check_output(['git', 'config', '--get', key])
        str_res = result.decode().strip()
        if key == 'remote.origin.url': return str_res.split('/')[-1][:-4]
        return str_res
    except subprocess.CalledProcessError:
        return None

# %% ../nbs/00_oai.ipynb 8
def _join_url(base, path):
    if not base.endswith('/'):
        base += '/'
    return urljoin(base, path)

# %% ../nbs/00_oai.ipynb 9
@call_parse
def create_openai_plugin_scaffolding(name:str=None, # the name of your app for the LLM, ex: `todo`.  By default, this is inferred from the name of your git repo.
                                     description:str=None, # the description of your application that will be read by the LLM.
                                     url='http://localhost:8000', # The url of your function endpoint. 
                                     email=None, # The email associated with your app.  By default this is inferred by git.
                                    ):
    "Generate minimal scaffolding for an OpenAI Plugin."
    # Prompt for the application description
    if not description: description = input("Enter the application description: ")
    if not url: 
        _url = input("Change the url for the app (default: http://localhost:8000): ")
        if url: url=_url

    # Try to infer name_for_model and contact_email from Git config
    if name: 
        name_for_model=name
    else:
        name_for_model = _query_git_config('remote.origin.url')
        if name_for_model: print(f"Inferred name of app for model and human as `{name_for_model.split('/')[-1]}`.  Modify .well-known/ai-plugin.json and main.py to change this.")
        else: name_for_model = input("Provide the name of your function for the model. (ex `todo`):")
        
    if not email:
        email = _query_git_config('user.email')
        if not email: email = input("Enter the contact email: ")
    
    # Write the ai-plugin.json file
    ai_plugin_info = {
        "schema_version": "v1",
        "name_for_human": name_for_model,  # Assuming the Git repo name is suitable for human-readable name
        "name_for_model": name_for_model,
        "description_for_human": description,
        "description_for_model": description,
        "auth": {
            "type": "none"
        },
        "api": {
            "type": "openapi",
            "url": f"{_join_url(url, 'openapi.yaml')}"
        },
        "logo_url": f"{_join_url(url, 'logo.png')}",
        "contact_email": email,
        "legal_info_url": "http://example.com/legal"
    }

    os.makedirs('.well-known', exist_ok=True)
    with open('.well-known/ai-plugin.json', 'w') as f:
        json.dump(ai_plugin_info, f, indent=4)

    # Write the main.py file
    with open('main.py', 'w') as f:
        f.write(
            'from fastapi import FastAPI, Response, Request\n'
            'from fastapi.responses import FileResponse\n'
            'import yaml\n\n'
            f'app = FastAPI(description="{description}", title="{name_for_model}")\n\n'  # Set description for FastAPI
            '@app.get("/some-route")\n'
            'async def example_route():\n'
            '    return {"message": "Hello World"}\n\n'
            '@app.get("/.well-known/ai-plugin.json", response_class=FileResponse)\n'
            'async def ai_plugin():\n'
            '    return FileResponse("my_plugin/.well-known/ai-plugin.json")\n\n'
            '@app.get("/logo.png", response_class=FileResponse)\n'
            'async def plugin_logo():\n'
            '    return FileResponse("my_plugin/static/logo.png")\n\n'
            '@app.get("/openapi.yaml")\n'
            'async def openapi(request: Request):\n'
            '    openapi_schema = app.openapi()\n'
            '    openapi_schema["servers"] = [{"url": f"https://{request.headers[\'Host\']}"}]\n'
            '    for p in ["/.well-known/ai-plugin.json", "/logo.png", "/openapi.yaml"]:\n'
            '        openapi_schema["paths"].pop(p)\n'
            '    return Response(content=yaml.dump(openapi_schema), media_type="text/yaml")\n\n'
        )

    print('Plugin scaffolding created in current directory.')
